<?php

  declare( strict_types = 1 );

  namespace Laodc\Dns;

  class Base
  {
    protected array $data_fields = [];

    /**
     * helper function to grab all data from child class
     *
     * @return array
     *        array of available data
     */
    public function toArray() : array
    {
      return $this->data_fields;
    }

    /**
     * @param string $field
     *        key of 
     *
     * @return mixed|null
     *        
     */
    public function get( string $field ) : mixed
    {
      return $this->data_fields[$field] ?? null;
    }

    /**
     * @param string|null $field
     *        key 
     * 
     * @param mixed $value
     * 
     * @return mixed
     */
    public function set( ?string $field, mixed $value = null ) : mixed
    {
      // reset array
      if( null === $field )
        return $this->data_fields = [];

      // if a record or domain is updating we run it through the formatter
      switch( get_class( $this ) )
      {
        case Record::class:
          return $this->data_fields[$field] = Validate::formatRecordData( $field, $value );
        case Domain::class:
          return $this->data_fields[$field] = Validate::formatDomainData( $field, $value );
        default:
          return $this->data_fields[$field] = $value;
      }
      
    }

    public function populate( array $func, array $data ) : void
    {
      foreach( $data as $k => $v )
        $this->data_fields[$k] = call_user_func( $func, $k, $v );
    }
  }
