<?php

  namespace Laodc\Dns;

  use Laodc\Database\Database;

  class Records extends Base
  {
    const TABLE_NAME = 'records';
    const MAX_TXT_LENGTH = 2048;
    const MAX_TXT_ELEM_LENGTH = 255;
    const MIN_TTL = 60;
    const DEFAULT_TTL = 1800;

    private int $group_size = 0;

    private Database $db;
    private Domain $domain;

    function __construct( Database $database, Domain $domain )
    {
      $this->db = $database;

      $this->domain = $domain;

      $this->set( 'records', [] );

      if( $domain->get( 'id' ) )
        $this->set( 'records', self::getRecordsById( $domain->get( 'id' ) ) );
    }

    public function getAll() : array
    {
      if( empty( $this->domain->get( 'id' ) ) )
        return [];

      return self::getRecordsById( $this->domain->get( 'id' ) );
    }

    public function getRecordsById( int $id ) : array
    {
      $sql = sprintf( '
        SELECT
          id, domain_id, name, type,
          data, pref, ttl, active
        FROM
          %s
        WHERE
          domain_id = %d
        ORDER BY
          CASE WHEN name LIKE \'%s\' THEN 0 ELSE 1 END ASC,
          name ASC, type ASC, 
          pref ASC, data ASC
        ',
        $this->db->escape( self::TABLE_NAME ),
        $id,
        $this->domain->get( 'fqdn' )
      );

      $records = [];

      foreach( $this->db->fetchAll( $sql ) as $record )
      {
        $record['active'] = Validate::toBool( $record['active'] );
        $records[] = new Record( $this->db, $this->domain, $record );
      }

      return $records;
    }

    public function getById( int $id ) :? Record
    {
      // if copy already exists in memory
      // search against that first, and if no
      // matches then check against database
      if( $this->get( 'records' ) )
      {
        foreach( $this->get( 'records' ) as $record )
        {
          if( $id == $record->get( 'id' ) )
            return $record;
        }
      }

      $sql = sprintf( '
        SELECT
          id, domain_id, name, type,
          data, pref, ttl, active
        FROM
          %s
        WHERE
          id = %d
        ',
        $this->db->escape( self::TABLE_NAME ),
        $id
      );

      if( [] === ( $result = $this->db->fetch( $sql ) ) )
        return null;

      return new Record( $this->db, $this->domain, $result );
    }

    public function new() :? Record
    {
      if( empty( $this->domain->get( 'id' ) ) )
        return null;

      $record = new Record( $this->db, $this->domain, [] );

      $records = $this->get( 'records' );
      $records[] = $record;
      $this->set( 'records', $records );

      return $record;
    }
  }